Index: common/common.c
===================================================================
--- common/common.c	(revision 564)
+++ common/common.c	(working copy)
@@ -62,6 +62,7 @@
     param->i_fps_num       = 25;
     param->i_fps_den       = 1;
     param->i_level_idc     = 51; /* as close to "unrestricted" as we can get */
+    param->i_slice_max     = 0; 
 
     /* Encoder parameters */
     param->i_frame_reference = 1;
Index: common/common.h
===================================================================
--- common/common.h	(revision 564)
+++ common/common.h	(working copy)
@@ -93,8 +93,7 @@
 }
 
 #define X264_BFRAME_MAX 16
-#define X264_SLICE_MAX 4
-#define X264_NAL_MAX (4 + X264_SLICE_MAX)
+#define X264_THREADS_MAX 4
 
 /****************************************************************************
  * Includes
@@ -268,13 +267,14 @@
     /* encoder parameters */
     x264_param_t    param;
 
-    x264_t *thread[X264_SLICE_MAX];
+    x264_t *thread[X264_THREADS_MAX];
 
     /* bitstream output */
     struct
     {
         int         i_nal;
-        x264_nal_t  nal[X264_NAL_MAX];
+        int         i_nal_bufsize;
+        x264_nal_t  * nal;
         int         i_bitstream;    /* size of p_bitstream */
         uint8_t     *p_bitstream;   /* will hold data for all nal */
         bs_t        bs;
Index: encoder/encoder.c
===================================================================
--- encoder/encoder.c	(revision 564)
+++ encoder/encoder.c	(working copy)
@@ -333,7 +333,7 @@
 
     if( h->param.i_threads == 0 )
         h->param.i_threads = x264_cpu_num_processors();
-    h->param.i_threads = x264_clip3( h->param.i_threads, 1, X264_SLICE_MAX );
+    h->param.i_threads = x264_clip3( h->param.i_threads, 1, X264_THREADS_MAX );
     h->param.i_threads = X264_MIN( h->param.i_threads, (h->param.i_height + 15) / 16 );
 #ifndef HAVE_PTHREAD
     if( h->param.i_threads > 1 )
@@ -540,7 +540,11 @@
     x264_reduce_fraction( &h->param.i_fps_num, &h->param.i_fps_den );
 
     /* Init x264_t */
-    h->out.i_nal = 0;
+    /* Create enough place for NAL-packets */
+    
+    /* We need 3 NALs for possible SPS, PPS and SEI and at least 1 for a slice */
+    h->out.i_nal_bufsize = 4;
+    h->out.nal = x264_malloc( h->out.i_nal_bufsize * sizeof( x264_nal_t ) );
     h->out.i_bitstream = X264_MAX( 1000000, h->param.i_width * h->param.i_height * 1.7
         * ( h->param.rc.i_rc_method == X264_RC_ABR ? pow( 0.5, h->param.rc.i_qp_min )
           : pow( 0.5, h->param.rc.i_qp_constant ) * X264_MAX( 1, h->param.rc.f_ip_factor )));
@@ -967,10 +971,8 @@
     int i_skip;
     int mb_xy;
     int i;
+	int i_pos_max;
 
-    /* init stats */
-    memset( &h->stat.frame, 0, sizeof(h->stat.frame) );
-
     /* Slice */
     x264_nal_start( h, h->i_nal_type, h->i_nal_ref_idc );
 
@@ -988,7 +990,11 @@
     h->mb.i_last_qp = h->sh.i_qp;
     h->mb.i_last_dqp = 0;
 
-    for( mb_xy = h->sh.i_first_mb, i_skip = 0; mb_xy < h->sh.i_last_mb; mb_xy++ )
+    mb_xy = h->sh.i_first_mb;
+    i_skip = 0;
+    i_pos_max = bs_pos(&h->out.bs) + (h->param.i_slice_max<<3);
+    
+    while ( mb_xy < h->sh.i_last_mb)
     {
         const int i_mb_y = mb_xy / h->sps->i_mb_width;
         const int i_mb_x = mb_xy % h->sps->i_mb_width;
@@ -1077,6 +1083,11 @@
 
         if( h->mb.b_variable_qp )
             x264_ratecontrol_mb(h, bs_pos(&h->out.bs) - mb_spos);
+
+        mb_xy++;
+
+        if ((h->param.i_slice_max > 0) && (bs_pos(&h->out.bs ) > i_pos_max))
+            break;
     }
 
     if( h->param.b_cabac )
@@ -1103,19 +1114,46 @@
     x264_nal_end( h );
 
     /* Compute misc bits */
-    h->stat.frame.i_misc_bits = bs_pos( &h->out.bs )
+    h->stat.frame.i_misc_bits += bs_pos( &h->out.bs )
                               + NALU_OVERHEAD * 8
                               - h->stat.frame.i_itex_bits
                               - h->stat.frame.i_ptex_bits
                               - h->stat.frame.i_hdr_bits;
 
+    /* set so that we know to do another slice */
+    h->sh.i_first_mb = mb_xy;
     return 0;
 }
 
+static int x264_slice_write_thread( x264_t *h )
+{
+    /* Not needed atm, but for possible future usage */
+    int i_return = 0;
+
+    /* init stats */
+    memset( &h->stat.frame, 0, sizeof(h->stat.frame) );
+
+    /* Run as long as the last MB isn't reached */
+    while( (0==i_return) && (h->sh.i_first_mb != h->sh.i_last_mb ))
+    {
+        i_return = x264_slice_write( h );
+
+        /* More NALs when needed */
+        if( h->out.i_nal >= h->out.i_nal_bufsize )
+        {
+            void * p_new;
+            h->out.i_nal_bufsize = h->out.i_nal + h->out.i_nal + 1;
+            p_new = x264_malloc( h->out.i_nal_bufsize );
+            memcpy( p_new, h->out.nal, h->out.i_nal*sizeof(x264_nal_t));
+            x264_free( h->out.nal );
+            h->out.nal = p_new;
+        }
+    }
+    return i_return;
+}
+
 static inline int x264_slices_write( x264_t *h )
 {
-    int i_frame_size;
-
 #if VISUALIZE
     if( h->param.b_visualize )
         x264_visualize_init( h );
@@ -1124,12 +1162,10 @@
     if( h->param.i_threads == 1 )
     {
         x264_ratecontrol_threads_start( h );
-        x264_slice_write( h );
-        i_frame_size = h->out.nal[h->out.i_nal-1].i_payload;
+        x264_slice_write_thread( h );
     }
     else
     {
-        int i_nal = h->out.i_nal;
         int i_bs_size = h->out.i_bitstream / h->param.i_threads;
         int i;
         /* duplicate contexts */
@@ -1139,43 +1175,56 @@
             if( i > 0 )
             {
                 memcpy( t, h, sizeof(x264_t) );
+
+                /* Create a temporary place for the NALs */
+                t->out.i_nal_bufsize = 4;
+                t->out.i_nal = 0;
+                t->out.nal = x264_malloc( h->out.i_nal_bufsize * sizeof(x264_nal_t) );
+
                 t->out.p_bitstream += i*i_bs_size;
                 bs_init( &t->out.bs, t->out.p_bitstream, i_bs_size );
                 t->i_thread_num = i;
             }
             t->sh.i_first_mb = (i    * h->sps->i_mb_height / h->param.i_threads) * h->sps->i_mb_width;
             t->sh.i_last_mb = ((i+1) * h->sps->i_mb_height / h->param.i_threads) * h->sps->i_mb_width;
-            t->out.i_nal = i_nal + i;
         }
         x264_ratecontrol_threads_start( h );
 
         /* dispatch */
 #ifdef HAVE_PTHREAD
         {
-            pthread_t handles[X264_SLICE_MAX];
+            pthread_t handles[X264_THREADS_MAX];
             for( i = 0; i < h->param.i_threads; i++ )
-                pthread_create( &handles[i], NULL, (void*)x264_slice_write, (void*)h->thread[i] );
+                pthread_create( &handles[i], NULL, (void*)x264_slice_write_thread, (void*)h->thread[i] );
             for( i = 0; i < h->param.i_threads; i++ )
                 pthread_join( handles[i], NULL );
         }
 #else
         for( i = 0; i < h->param.i_threads; i++ )
-            x264_slice_write( h->thread[i] );
+            x264_slice_write_thread( h->thread[i] );
 #endif
 
         /* merge contexts */
-        i_frame_size = h->out.nal[i_nal].i_payload;
         for( i = 1; i < h->param.i_threads; i++ )
         {
             int j;
             x264_t *t = h->thread[i];
-            h->out.nal[i_nal+i] = t->out.nal[i_nal+i];
-            i_frame_size += t->out.nal[i_nal+i].i_payload;
+
+            /* Copy the thread's NALs back into the main thread. */
+            if ( h->out.i_nal + t->out.i_nal >= h->out.i_nal_bufsize )
+            {
+                h->out.i_nal_bufsize = h->out.i_nal + t->out.i_nal + 1;
+                h->out.nal = x264_realloc( h->out.nal, h->out.i_nal_bufsize * sizeof(x264_nal_t) );
+            }
+            memcpy( &h->out.nal[h->out.i_nal], t->out.nal, t->out.i_nal * sizeof(x264_nal_t) );
+            h->out.i_nal += t->out.i_nal;
+
+            x264_free( t->out.nal );
+
             // all entries in stat.frame are ints
             for( j = 0; j < sizeof(h->stat.frame) / sizeof(int); j++ )
                 ((int*)&h->stat.frame)[j] += ((int*)&t->stat.frame)[j];
         }
-        h->out.i_nal = i_nal + h->param.i_threads;
     }
 
 #if VISUALIZE
@@ -1186,7 +1235,11 @@
     }
 #endif
 
-    return i_frame_size;
+    // these two are functionally equivalent
+    return ( h->stat.frame.i_hdr_bits
+           + h->stat.frame.i_itex_bits
+           + h->stat.frame.i_ptex_bits
+           + h->stat.frame.i_hdr_bits ) >> 3;
 }
 
 /****************************************************************************
@@ -1872,5 +1925,6 @@
     x264_free( h->out.p_bitstream );
     for( i = 1; i < h->param.i_threads; i++ )
         x264_free( h->thread[i] );
+	x264_free( h->out.nal );
     x264_free( h );
 }
Index: x264.c
===================================================================
--- x264.c	(revision 564)
+++ x264.c	(working copy)
@@ -165,6 +165,8 @@
     H1( "      --nf                    Disable loop filter\n" );
     H0( "  -f, --filter <alpha:beta>   Loop filter AlphaC0 and Beta parameters [%d:%d]\n",
                                        defaults->i_deblocking_filter_alphac0, defaults->i_deblocking_filter_beta );
+    H0( "      --max-slicesize <integer>  Limit the size of each slice (in bytes) (0 => unlimited)\n");
+
     H0( "\n" );
     H0( "Ratecontrol:\n" );
     H0( "\n" );
@@ -338,6 +340,7 @@
 #define OPT_PROGRESS 261
 #define OPT_VISUALIZE 262
 #define OPT_LONGHELP 263
+#define OPT_SLICE_MAX 264
 
         static struct option long_options[] =
         {
@@ -398,6 +401,7 @@
             { "zones",   required_argument, NULL, 0 },
             { "qpfile",  required_argument, NULL, OPT_QPFILE },
             { "threads", required_argument, NULL, 0 },
+            { "max-slicesize", required_argument, NULL, OPT_SLICE_MAX },
             { "thread-input", no_argument,  NULL, OPT_THREAD_INPUT },
             { "no-psnr", no_argument,       NULL, 0 },
             { "no-ssim", no_argument,       NULL, 0 },
@@ -499,6 +503,9 @@
                 param->i_scenecut_threshold = -1;
                 param->b_bframe_adaptive = 0;
                 break;
+            case OPT_SLICE_MAX:
+                param->i_slice_max = atoi(optarg);
+                break;
             case OPT_THREAD_INPUT:
                 b_thread_input = 1;
                 break;
Index: x264.h
===================================================================
--- x264.h	(revision 564)
+++ x264.h	(working copy)
@@ -259,6 +259,8 @@
     int b_aud;                  /* generate access unit delimiters */
     int b_repeat_headers;       /* put SPS/PPS before each keyframe */
     int i_sps_id;               /* SPS and PPS id number */
+    int i_slice_max; /* maximum slice size in bytes, split slices if larger than this many bytes (0 => unlimited)  */
+
 } x264_param_t;
 
 typedef struct {